import type { RequestHandler } from '@sveltejs/kit';
import { getSession } from '$lib/server/session';
import { Permissions } from '$lib/permissions';
import { FeedbackListSchema } from './schema';
import { Feedback, sequelize, User } from '$lib/server/database';
import { type Attributes, Op } from 'sequelize';

export const POST = (async ({ request, cookies }) => {
	if (getSession(cookies, { permissions: [Permissions.Feedback] }) == null) {
		return new Response(null, {
			status: 401
		});
	}

	const parseResult = await FeedbackListSchema.safeParseAsync(await request.json());
	if (!parseResult.success) {
		return new Response(null, {
			status: 400
		});
	}
	const data = parseResult.data;

	const feedbackFindOptions: Attributes<Feedback> = {
		content: { [Op.not]: null }
	};
	if (data.event) Object.assign(feedbackFindOptions, { event: { [Op.like]: `%${data.event}%` } });
	// prettier-ignore
	if (data.content) Object.assign(feedbackFindOptions, { content: { [Op.like]: `%${data.content}%` } });
	if (data.username)
		Object.assign(feedbackFindOptions, {
			user_id: await User.findAll({
				attributes: ['id'],
				where: { username: { [Op.like]: `%${data.username}%` } }
			}).then((users) => users.map((user) => user.id))
		});
	if (data.hash) Object.assign(feedbackFindOptions, { url_hash: data.hash });

	const feedback = await Feedback.findAll({
		where: feedbackFindOptions,
		// eslint-disable-next-line @typescript-eslint/ban-ts-comment
		// @ts-ignore
		attributes: data.preview
			? {
					exclude: ['content'],
					include: [
						[sequelize.literal('SUBSTR(content, 1, 50)'), 'content'],
						[sequelize.literal('LENGTH(content) > 50'), 'content_stripped']
					]
				}
			: undefined,
		include: { model: User, as: 'user' },
		offset: data.hash ? 0 : data.from || 0,
		limit: data.hash ? 1 : data.limit || 100
	});

	return new Response(JSON.stringify(feedback));
}) satisfies RequestHandler;
